@@ -3,11 +3,20 @@
 split.labs.wrapper <- function(x, split.fun, split.fun.name,
                                split.prefix, split.suffix,
                                right.split.prefix, right.split.suffix,
-                               type, clip.left.labs, clip.right.labs, xflip, digits,
-                               varlen, faclen, facsep, eq, lt, ge)
+                               type, clip.facs,
+                               clip.left.labs, clip.right.labs, xflip,
+                               digits, varlen, faclen, roundint, trace,
+                               facsep, eq, lt, ge)
 {
-    labs <- internal.split.labs(x, type, clip.left.labs, clip.right.labs, xflip,
-                                digits, varlen, faclen, facsep, eq, lt, ge,
+    logical.eq <- eq
+    if(clip.facs)
+        eq <- "|" # special value used as a flag
+
+    labs <- internal.split.labs(x, type,
+                                digits, varlen, faclen, roundint,
+                                clip.facs, clip.left.labs, clip.right.labs, xflip,
+                                trace,
+                                facsep, eq, logical.eq, lt, ge,
                                 split.prefix, right.split.prefix,
                                 split.suffix, right.split.suffix)
 
@@ -22,8 +31,11 @@ split.labs.wrapper <- function(x, split.fun, split.fun.name,
 }
 # Modified version of labels.rpart.
 # This uses format0 instead of formatg and has various other extensions.
-internal.split.labs <- function(x, type, clip.left.labs, clip.right.labs, xflip,
-                                digits, varlen, faclen, facsep, eq, lt, ge,
+internal.split.labs <- function(x, type,
+                                digits, varlen, faclen, roundint,
+                                clip.facs, clip.left.labs, clip.right.labs, xflip,
+                                trace,
+                                facsep, eq, logical.eq, lt, ge,
                                 split.prefix, right.split.prefix,
                                 split.suffix, right.split.suffix)
 {
@@ -32,6 +44,7 @@ internal.split.labs <- function(x, type, clip.left.labs, clip.right.labs, xflip,
         return("root")   # NOTE: return
     is.leaf <- is.leaf(frame)
     split.var.names <- frame$var[!is.leaf]  # variable names for the (primary) splits
+
     split.var.names <- as.character(split.var.names) # factor levels to character
     clip.left.labs  <- recycle(clip.left.labs,  split.var.names)
     clip.right.labs <- recycle(clip.right.labs, split.var.names)
@@ -42,39 +55,61 @@ internal.split.labs <- function(x, type, clip.left.labs, clip.right.labs, xflip,
 
     split <- get.lsplit.rsplit(x, isplit, split.var.names,
                                type, clip.left.labs, clip.right.labs, xflip,
-                               digits, faclen, facsep, eq, lt, ge)
+                               digits, faclen, roundint, trace,
+                               facsep, eq, logical.eq, lt, ge)
 
     # We now have something like this:
     #    split.var.names:   sex   age     pclass     sibsp   pclass
     #    split$lsplit:      mal   >=9.5   =2nd,3rd   >=2.5   =3rd
     #    split$rsplit:      fml   <9.5    =1st       <2.5    =1st,2nd
 
-    paste.split.labs(frame, split.var.names, split$lsplit, split$rsplit,
-                     type, clip.left.labs, clip.right.labs, xflip, varlen,
+    if(clip.facs) {
+        is.eq.l <- substr(split$lsplit, 1, 1) == "|" # special value used a flag
+        is.eq.r <- substr(split$rsplit, 1, 1) == "|"
+        split.var.names[is.eq.l | is.eq.r] <- ""                     # drop var name
+        split$lsplit[is.eq.l] <- substring(split$lsplit[is.eq.l], 2) # drop leading |
+        split$rsplit[is.eq.r] <- substring(split$rsplit[is.eq.r], 2)
+    }
+    paste.split.labs(frame,
+                     split.var.names, split$lsplit, split$rsplit,
+                     type, clip.facs,
+                     clip.left.labs, clip.right.labs, xflip, varlen,
                      split.prefix, right.split.prefix,
                      split.suffix, right.split.suffix)
 }
 get.lsplit.rsplit <- function(x, isplit, split.var.names,
                               type, clip.left.labs, clip.right.labs, xflip,
-                              digits, faclen, facsep, eq, lt, ge)
+                              digits, faclen, roundint, trace,
+                              facsep, eq, logical.eq, lt, ge)
 {
     frame <- x$frame
     is.leaf <- is.leaf(frame)
-    ncat  <- x$splits[isplit, "ncat"]
+    splits <- tweak.splits(x, roundint, digits, trace)
+    ncat  <- splits[isplit, "ncat"]
     lsplit <- rsplit <- character(length=length(isplit))
     is.con <- ncat <= 1             # continuous vars (a logical vector)
+    is.logical <- get.is.logical(x, splits, trace)[isplit]
     if(any(is.con)) {               # any continuous vars?
-        cutpoint <- format0(x$splits[isplit[is.con], "index"], digits)
+        cut <- splits[isplit[is.con], "index"]
+        formatted.cut <- format0(cut, digits)
         is.less.than <- ncat < 0
-        lsplit[is.con] <- paste0(ifelse(is.less.than, lt, ge)[is.con], cutpoint)
-        rsplit[is.con] <- paste0(ifelse(is.less.than, ge, lt)[is.con], cutpoint)
+        lsplit[is.con] <- paste0(ifelse(is.less.than, lt, ge)[is.con], formatted.cut)
+        rsplit[is.con] <- paste0(ifelse(is.less.than, ge, lt)[is.con], formatted.cut)
+
+        # print logical as "Survived = 1" not "Survived >= .5"
+        if(!anyNA(is.logical) && any(is.logical)) {
+            eq0 <- paste0(logical.eq, "0")
+            eq1 <- paste0(logical.eq, "1")
+            lsplit[is.logical] <- paste0(ifelse(is.less.than, eq0, eq1)[is.logical])
+            rsplit[is.logical] <- paste0(ifelse(is.less.than, eq1, eq0)[is.logical])
+        }
     }
     is.cat <- ncat > 1              # categorical variables (a logical vector)
     if(any(is.cat)) {               # any categorical variables?
         # jrow is the row numbers of factors within lsplit and rsplit
         # cindex is the index on the "xlevels" list
         jrow <- seq_along(ncat)[is.cat]
-        crow <- x$splits[isplit[is.cat], "index"] # row number in csplit
+        crow <- splits[isplit[is.cat], "index"] # row number in csplit
         xlevels <- attr(x, "xlevels")
         cindex <- match(split.var.names, names(xlevels))[is.cat]
         # decide if we must add a "=" prefix
@@ -89,16 +124,65 @@ get.lsplit.rsplit <- function(x, isplit, split.var.names,
             left  <- (1:length(splits))[splits == 1]
             right <- (1:length(splits))[splits == 3]
             collapse <- if(faclen==1) "" else facsep
-            lsplit[j] <- paste(node.xlevels[left],  collapse=collapse)
+            lsplit[j] <- paste(node.xlevels[left],   collapse=collapse)
+            rsplit[j] <- paste0(node.xlevels[right], collapse=collapse)
             if(paste.left.eq[i])
                 lsplit[j] <- paste0(eq, lsplit[j])
-            rsplit[j] <- paste0(node.xlevels[right], collapse=collapse)
             if(paste.right.eq[i])
                 rsplit[j] <- paste0(eq, rsplit[j])
         }
     }
     list(lsplit=lsplit, rsplit=rsplit)
 }
+# If roundint is TRUE then round up the entries in splits$index for
+# variables that are integral. (All values in the input data for the
+# variable must be integral, not just the entry in splits$index.)
+#
+# Add verysmall to all other cuts where verysmall is something like 1e-10.
+# This is so format(cut, digits=digits) always rounds up (rather than the
+# default behaviour of format which is to round to even the last digit is
+# even).
+# This gives more easily interpretable results, especially because split
+# cuts ending in .5 are common for integer predictors.
+# Thus "nbr.family.members < 2.5" is now rounded to "nbr.family.members < 3"
+# (rather than "nbr.family.members < 2", which was misleading).
+#
+# e.g. format(1234.5,       digits=2) is 1234   rounds down (not what we want)
+#      format(1235.5,       digits=2) is 1236   rounds up
+#      format(1234.5+1e-10, digits=2) is 1235   rounds up
+#      format(1235.5+1e-10, digits=2) is 1236   rounds up
+#
+# Note that if roundint is FALSE then we still bump all entries by verysmall.
+
+tweak.splits <- function(obj, roundint, digits, trace)
+{
+    exp10 <- function(x) exp(x * log(10)) # e.g. exp10(-3) = 1e-3
+    #--- tweak.splits starts here ---
+    verysmall <- exp10(-abs(digits) - 8)
+    splits <- obj$splits
+    splits[,"index"] <- splits[,"index"] + verysmall
+    # Because rpart.model.frame() may fail or give warnings, we do
+    # processing here only if the roundint argument is TRUE.
+    if(roundint) {
+        is.roundint <- get.is.roundint(obj) # vector of bools
+        if(anyNA(is.roundint)) # rpart.model.frame failed?
+            return(splits)
+        if(trace >= 2) {
+            varnames <- names(is.roundint)[is.roundint]
+            if(length(varnames))
+                cat("will apply roundint to the following variables:",
+                    varnames, "\n")
+            else
+                cat("will apply roundint to no variables\n")
+        }
+        for(varname in names(is.roundint)) if(is.roundint[varname]) {
+            i <- rownames(splits) %in% varname
+            if(length(i) > 0)
+                splits[,"index"][i] <- ceiling(splits[,"index"][i] - verysmall)
+        }
+    }
+    splits
+}
 # Paste the various constituents to create the split labels vector.
 # On entry we have something like this:
 #    split.var.names sex   age     pclass     sibsp   pclass
@@ -106,7 +190,8 @@ get.lsplit.rsplit <- function(x, isplit, split.var.names,
 #    rsplit:         fml   <9.5    =1st       <2.5    =1st,2nd
 
 paste.split.labs <- function(frame, split.var.names, lsplit, rsplit,
-                             type, clip.left.labs, clip.right.labs, xflip, varlen,
+                             type, clip.facs,
+                             clip.left.labs, clip.right.labs, xflip, varlen,
                              split.prefix, right.split.prefix,
                              split.suffix, right.split.suffix)
 {
@@ -116,6 +201,7 @@ paste.split.labs <- function(frame, split.var.names, lsplit, rsplit,
     parent <- match(nodes %/% 2, nodes[!is.leaf])
     split.var.names <- my.abbreviate(split.var.names, varlen)
     left.names <- right.names <- split.var.names
+
     if(is.fancy(type)) {
         if(xflip)
             right.names[clip.left.labs] <- ""
@@ -134,10 +220,10 @@ paste.split.labs <- function(frame, split.var.names, lsplit, rsplit,
     # the heart of this function
 
     labs  <- paste0(split.prefix, left.names[parent], lsplit[parent], split.suffix)
-    labs[1] <- "root" # was NA NA
     labs[is.right] <- paste0(right.split.prefix,
                              right.names[parent],
                              rsplit[parent],
                              right.split.suffix)[is.right]
+    labs[1] <- "root" # was "NANA" because of above paste0
     labs
 }
